@@ -7,27 +7,25 @@
 # All rights reserved.
 # 
 # Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
-#     * Redistributions of source code must retain the above copyright
-#       notice, this list of conditions and the following disclaimer.
-#     * Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in the
-#       documentation and/or other materials provided with the distribution.
-#     * Neither the name of the <organization> nor the
-#       names of its contributors may be used to endorse or promote products
-#       derived from this software without specific prior written permission.
+# modification, are permitted provided that the following conditions are met: 
+# 
+# 1. Redistributions of source code must retain the above copyright notice, this
+# list of conditions and the following disclaimer. 
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+# this list of conditions and the following disclaimer in the documentation
+# and/or other materials provided with the distribution. 
 # 
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 # ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-# DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
-# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
+# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 # (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 # LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 # ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-#
+# 
 ##############################################################################
 
 """
@@ -38,18 +36,10 @@ Code repository: https://code.launchpad.net/~niv-openerp/openerp-client-lib/trun
 """
 
 import xmlrpclib
-import logging 
-import socket
-
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
-
-try:
-    import cStringIO as StringIO
-except ImportError:
-    import StringIO
+import logging
+import json
+import urllib2
+import random
 
 _logger = logging.getLogger(__name__)
 
@@ -63,14 +53,13 @@ class Connector(object):
 
     __logger = _getChildLogger(_logger, 'connector')
 
-    def __init__(self, hostname, port):
+    def get_service(self, service_name):
         """
-        Initilize by specifying an hostname and a port.
-        :param hostname: Host name of the server.
-        :param port: Port for the connection to the server.
+        Returns a Service instance to allow easy manipulation of one of the services offered by the remote server.
+
+        :param service_name: The name of the service.
         """
-        self.hostname = hostname
-        self.port = port
+        return Service(self, service_name)
 
 class XmlRPCConnector(Connector):
     """
@@ -86,111 +75,84 @@ class XmlRPCConnector(Connector):
         :param hostname: The hostname of the computer holding the instance of OpenERP.
         :param port: The port used by the OpenERP instance for XMLRPC (default to 8069).
         """
-        Connector.__init__(self, hostname, port)
-        self.url = 'http://%s:%d/xmlrpc' % (self.hostname, self.port)
+        self.url = 'http://%s:%d/xmlrpc' % (hostname, port)
 
     def send(self, service_name, method, *args):
         url = '%s/%s' % (self.url, service_name)
         service = xmlrpclib.ServerProxy(url)
         return getattr(service, method)(*args)
 
-class NetRPC_Exception(Exception):
+class XmlRPCSConnector(XmlRPCConnector):
     """
-    Exception for NetRPC errors.
+    A type of connector that uses the secured XMLRPC protocol.
     """
-    def __init__(self, faultCode, faultString):
-        self.faultCode = faultCode
-        self.faultString = faultString
-        self.args = (faultCode, faultString)
+    PROTOCOL = 'xmlrpcs'
 
-class NetRPC(object):
+    __logger = _getChildLogger(_logger, 'connector.xmlrpcs')
+
+    def __init__(self, hostname, port=8069):
+        super(XmlRPCSConnector, self).__init__(hostname, port)
+        self.url = 'https://%s:%d/xmlrpc' % (hostname, port)
+
+class JsonRPCException(Exception):
+    def __init__(self, error):
+         self.error = error
+    def __str__(self):
+         return repr(self.error)
+
+def json_rpc(url, fct_name, params):
+    data = {
+        "jsonrpc": "2.0",
+        "method": fct_name,
+        "params": params,
+        "id": random.randint(0, 1000000000),
+    }
+    req = urllib2.Request(url=url, data=json.dumps(data), headers={
+        "Content-Type":"application/json",
+    })
+    result = urllib2.urlopen(req)
+    result = json.load(result)
+    if result.get("error", None):
+        raise JsonRPCException(result["error"])
+    return result["result"]
+
+class JsonRPCConnector(Connector):
     """
-    Low level class for NetRPC protocol.
+    A type of connector that uses the JsonRPC protocol.
     """
-    def __init__(self, sock=None):
-        if sock is None:
-            self.sock = socket.socket(
-            socket.AF_INET, socket.SOCK_STREAM)
-        else:
-            self.sock = sock
-        self.sock.settimeout(120)
-    def connect(self, host, port=False):
-        if not port:
-            buf = host.split('//')[1]
-            host, port = buf.split(':')
-        self.sock.connect((host, int(port)))
-
-    def disconnect(self):
-        self.sock.shutdown(socket.SHUT_RDWR)
-        self.sock.close()
-
-    def mysend(self, msg, exception=False, traceback=None):
-        msg = pickle.dumps([msg,traceback])
-        size = len(msg)
-        self.sock.send('%8d' % size)
-        self.sock.send(exception and "1" or "0")
-        totalsent = 0
-        while totalsent < size:
-            sent = self.sock.send(msg[totalsent:])
-            if sent == 0:
-                raise RuntimeError, "socket connection broken"
-            totalsent = totalsent + sent
-
-    def myreceive(self):
-        buf=''
-        while len(buf) < 8:
-            chunk = self.sock.recv(8 - len(buf))
-            if chunk == '':
-                raise RuntimeError, "socket connection broken"
-            buf += chunk
-        size = int(buf)
-        buf = self.sock.recv(1)
-        if buf != "0":
-            exception = buf
-        else:
-            exception = False
-        msg = ''
-        while len(msg) < size:
-            chunk = self.sock.recv(size-len(msg))
-            if chunk == '':
-                raise RuntimeError, "socket connection broken"
-            msg = msg + chunk
-        msgio = StringIO.StringIO(msg)
-        unpickler = pickle.Unpickler(msgio)
-        unpickler.find_global = None
-        res = unpickler.load()
-
-        if isinstance(res[0],Exception):
-            if exception:
-                raise NetRPC_Exception(str(res[0]), str(res[1]))
-            raise res[0]
-        else:
-            return res[0]
-
-class NetRPCConnector(Connector):
-    """
-    A type of connector that uses the NetRPC protocol.
-    """
-
-    PROTOCOL = 'netrpc'
+    PROTOCOL = 'jsonrpc'
     
-    __logger = _getChildLogger(_logger, 'connector.netrpc')
+    __logger = _getChildLogger(_logger, 'connector.jsonrpc')
 
-    def __init__(self, hostname, port=8070):
+    def __init__(self, hostname, port=8069):
         """
         Initialize by specifying the hostname and the port.
         :param hostname: The hostname of the computer holding the instance of OpenERP.
-        :param port: The port used by the OpenERP instance for NetRPC (default to 8070).
+        :param port: The port used by the OpenERP instance for JsonRPC (default to 8069).
         """
-        Connector.__init__(self, hostname, port)
+        self.url = 'http://%s:%d/jsonrpc' % (hostname, port)
 
     def send(self, service_name, method, *args):
-        socket = NetRPC()
-        socket.connect(self.hostname, self.port)
-        socket.mysend((service_name, method, )+args)
-        result = socket.myreceive()
-        socket.disconnect()
-        return result
+        return json_rpc(self.url, "call", {"service": service_name, "method": method, "args": args})
+
+class JsonRPCSConnector(Connector):
+    """
+    A type of connector that uses the JsonRPC protocol.
+    """
+    PROTOCOL = 'jsonrpcs'
+    
+    __logger = _getChildLogger(_logger, 'connector.jsonrpc')
+
+    def __init__(self, hostname, port=8069):
+        """
+        Initialize by specifying the hostname and the port.
+        :param hostname: The hostname of the computer holding the instance of OpenERP.
+        :param port: The port used by the OpenERP instance for JsonRPC (default to 8069).
+        """
+        self.url = 'https://%s:%d/jsonrpc' % (hostname, port)
+
+    def send(self, service_name, method, *args):
+        return json_rpc(self.url, "call", {"service": service_name, "method": method, "args": args})
 
 class Service(object):
     """
@@ -203,7 +165,7 @@ class Service(object):
         """
         self.connector = connector
         self.service_name = service_name
-        self.__logger = _getChildLogger(_getChildLogger(_logger, 'service'),service_name)
+        self.__logger = _getChildLogger(_getChildLogger(_logger, 'service'),service_name or "")
         
     def __getattr__(self, method):
         """
@@ -246,6 +208,7 @@ class Connection(object):
         self.connector = connector
 
         self.set_login_info(database, login, password, user_id)
+        self.user_context = None
 
     def set_login_info(self, database, login, password, user_id=None):
         """
@@ -274,12 +237,21 @@ class Connection(object):
             return
         
         if not self.database or not self.login or self.password is None:
-            raise AuthenticationError("Creditentials not provided")
-        
+            raise AuthenticationError("Credentials not provided")
+
+        # TODO use authenticate instead of login
         self.user_id = self.get_service("common").login(self.database, self.login, self.password)
         if not self.user_id:
             raise AuthenticationError("Authentication failure")
         self.__logger.debug("Authenticated with user id %s", self.user_id)
+        
+    def get_user_context(self):
+        """
+        Query the default context of the user.
+        """
+        if not self.user_context:
+            self.user_context = self.get_model('res.users').context_get()
+        return self.user_context
     
     def get_model(self, model_name):
         """
@@ -297,7 +269,7 @@ class Connection(object):
 
         :param service_name: The name of the service.
         """
-        return Service(self.connector, service_name)
+        return self.connector.get_service(service_name)
 
 class AuthenticationError(Exception):
     """
@@ -318,7 +290,7 @@ class Model(object):
         """
         self.connection = connection
         self.model_name = model_name
-        self.__logger = _getChildLogger(_getChildLogger(_logger, 'object'), model_name)
+        self.__logger = _getChildLogger(_getChildLogger(_logger, 'object'), model_name or "")
 
     def __getattr__(self, method):
         """
@@ -326,25 +298,25 @@ class Model(object):
 
         :param method: The method for the linked model (search, read, write, unlink, create, ...)
         """
-        def proxy(*args):
+        def proxy(*args, **kw):
             """
             :param args: A list of values for the method
             """
             self.connection.check_login(False)
             self.__logger.debug(args)
-            result = self.connection.get_service('object').execute(
+            result = self.connection.get_service('object').execute_kw(
                                                     self.connection.database,
                                                     self.connection.user_id,
                                                     self.connection.password,
                                                     self.model_name,
                                                     method,
-                                                    *args)
+                                                    args, kw)
             if method == "read":
                 if isinstance(result, list) and len(result) > 0 and "id" in result[0]:
                     index = {}
                     for r in result:
                         index[r['id']] = r
-                    result = [index[x] for x in args[0]]
+                    result = [index[x] for x in args[0] if x in index]
             self.__logger.debug('result: %r', result)
             return result
         return proxy
@@ -362,33 +334,38 @@ class Model(object):
         :return: A list of dictionaries containing all the specified fields.
         """
         record_ids = self.search(domain or [], offset, limit or False, order or False, context or {})
+        if not record_ids: return []
         records = self.read(record_ids, fields or [], context or {})
         return records
 
-def get_connector(hostname, protocol="xmlrpc", port="auto"):
+def get_connector(hostname=None, protocol="xmlrpc", port="auto"):
     """
-    A shortcut method to easily create a connector to a remote server using XMLRPC or NetRPC.
+    A shortcut method to easily create a connector to a remote server using XMLRPC.
 
     :param hostname: The hostname to the remote server.
-    :param protocol: The name of the protocol, must be "xmlrpc" or "netrpc".
+    :param protocol: The name of the protocol, must be "xmlrpc" or "xmlrpcs".
     :param port: The number of the port. Defaults to auto.
     """
     if port == 'auto':
-        port = 8069 if protocol=="xmlrpc" else 8070
+        port = 8069
     if protocol == "xmlrpc":
         return XmlRPCConnector(hostname, port)
-    elif protocol == "netrpc":
-        return NetRPCConnector(hostname, port)
+    elif protocol == "xmlrpcs":
+        return XmlRPCSConnector(hostname, port)
+    if protocol == "jsonrpc":
+        return JsonRPCConnector(hostname, port)
+    elif protocol == "jsonrpcs":
+        return JsonRPCSConnector(hostname, port)
     else:
-        raise ValueError("You must choose xmlrpc or netrpc")
+        raise ValueError("You must choose xmlrpc or xmlrpcs")
 
-def get_connection(hostname, protocol="xmlrpc", port='auto', database=None,
+def get_connection(hostname=None, protocol="xmlrpc", port='auto', database=None,
                  login=None, password=None, user_id=None):
     """
     A shortcut method to easily create a connection to a remote OpenERP server.
 
     :param hostname: The hostname to the remote server.
-    :param protocol: The name of the protocol, must be "xmlrpc" or "netrpc".
+    :param protocol: The name of the protocol, must be "xmlrpc" or "xmlrpcs".
     :param port: The number of the port. Defaults to auto.
     :param connector: A valid Connector instance to send messages to the remote server.
     :param database: The name of the database to work on.
